#!/usr/bin/env python3

import argparse
import sys
import json
import yaml

from aws_policy_generator import mappings
from aws_policy_generator import auto_shortener
from aws_iam_utils.generator import generate_policy_for_service
from aws_iam_utils.generator import generate_policy_for_service_arn_type
from aws_iam_utils.generator import generate_full_policy_for_service
from aws_iam_utils.constants import READ, WRITE, LIST, TAGGING, PERMISSIONS, ALL_ACCESS_LEVELS
from aws_iam_utils.combiner import collapse_policy_statements
from aws_iam_utils.simplifier import simplify_policy
from aws_iam_utils.util import create_policy
from aws_iam_utils.util import statement

from policyuniverse.expander_minimizer import minimize_policy

# YAML example:
#
# policies:
#   - service: iam
#     resource_type: policy
#     access_level: read
#   - service: s3
#     access_level: all
#   - action: ec2:DescribeRegions
#
# The access_level key can be list, read, write, tagging, permissions or all.


def generate_from_yaml(yamlInput, minimize=False, compact=False, auto_shorten=False, max_length=6144):
    yamlData = yaml.safe_load(yamlInput)

    policies = []  # list of policies that we'll collapse together at the end

    for yamlDataPolicy in yamlData['policies']:
        if 'action' in yamlDataPolicy:
            action_name = yamlDataPolicy['action']
            resource = yamlDataPolicy.get('resource', '*')
            policies.append(create_policy(statement(actions=action_name, resource=resource)))

        elif 'service' in yamlDataPolicy:
            service_name = yamlDataPolicy['service']
            resource_type = yamlDataPolicy.get('resource_type', '*')
            access_level = yamlDataPolicy.get('access_level', 'read')

            # attempt to map access_level
            access_levels = mappings.ACCESS_LEVELS_MAPPINGS[access_level]

            if resource_type == '*':
                if access_level == 'all':
                    policies.append(generate_full_policy_for_service(service_name))
                else:
                    policies.append(generate_policy_for_service(service_name, access_levels))

            else:
                policies.append(generate_policy_for_service_arn_type(service_name, resource_type, access_levels))

        else:
            raise ValueError(f'invalid input: {yamlDataPolicy}, must have "service" or "action" key')

    policy = simplify_policy(collapse_policy_statements(*policies))

    return policy



import json
import sys

from aws_policy_generator.args_generator import generate_from_args
from aws_policy_generator.yaml_generator import generate_from_yaml
from aws_policy_generator._internal import argparser
from aws_iam_utils.combiner import collapse_policy_statements

def main(args=None):
    if args is None:
        args = sys.argv[1:]

    args_namespace = argparser.parser.parse_args(args)

    policies = []

    if args_namespace.file:
        with open(args_namespace.file, 'r') as f:
            policies.append(generate_from_yaml(f))

    policies.append(generate_from_args(args_namespace))

    policy = collapse_policy_statements(*policies)

    if args_namespace.auto_shorten:
        policy_str = auto_shortener.auto_shorten_policy(policy, minimize=args_namespace.minimize, compact=args_namespace.compact, max_length=args_namespace.max_length)
    else:
        policy_str = json.dumps(policy, indent=2)

    return policy_str



from aws_iam_utils.constants import READ, WRITE, LIST, TAGGING, PERMISSIONS, ALL_ACCESS_LEVELS

FULL_ACCESS = '*'

ACCESS_LEVELS_MAPPINGS = {
  'list': [ LIST ],
  'read': [ LIST, READ ],
  'write': [ LIST, READ, WRITE ],
  'tagging': [ TAGGING ],
  'permissions': [ PERMISSIONS ],
  'all': FULL_ACCESS,
}



import json

from policyuniverse.expander_minimizer import minimize_policy

# auto-shorten: tuple of (minimize, compact)
AUTO_SHORTEN_ATTEMPT = [
    (False, False),
    (True, False),
    (True, True),
]

def auto_shorten_policy(policy: dict, minimize: bool = False, compact: bool = False, max_length: int = 6144) -> str:
  auto_shorten_attempts = AUTO_SHORTEN_ATTEMPT
  current_minimize = minimize
  current_compact = compact

  while True:
      if current_compact:
          policy = json.dumps(policy)
      
      if current_minimize:
          policy = json.dumps(minimize_policy(json.loads(policy)))

      policy_length = len(policy)
      if policy_length > max_length:
          if auto_shorten:
              if len(auto_shorten_attempts) == 0:
                  raise ValueError(f"the generated policy is {policy_length} characters, which is larger than the maximum {max_length} characters allowed. this policy is too long even after auto-shortening. try specifying fewer arguments")

              current_minimize, current_compact = auto_shorten_attempts.pop(0)
              # loop again with new current_* args

          else:
              raise ValueError(f"the generated policy is {policy_length} characters, which is larger than the maximum {max_length} characters allowed. try using --compact, --minimize, or specifying fewer arguments")

      else:
          break

  if policy_length > max_length:
      raise ValueError(f"the generated policy is {policy_length} characters, which is larger than the maximum {max_length} characters allowed. try using --compact, --minimize, or specifying fewer arguments")

  return policy




#!/usr/bin/env python3

import sys
import json
import argparse


from aws_policy_generator._internal import argparser
from aws_policy_generator import mappings
from aws_policy_generator import auto_shortener
from aws_iam_utils.generator import generate_policy_for_service
from aws_iam_utils.generator import generate_policy_for_service_arn_type
from aws_iam_utils.generator import generate_full_policy_for_service
from aws_iam_utils.constants import READ, WRITE, LIST, ALL_ACCESS_LEVELS
from aws_iam_utils.combiner import collapse_policy_statements
from aws_iam_utils.simplifier import simplify_policy
from aws_iam_utils.util import create_policy
from aws_iam_utils.util import statement

from policyuniverse.expander_minimizer import minimize_policy

def generate_from_args(args_namespace):
    policies = []  # list of policies that we'll mcollapse together at the end

    for item in [
            {
                'args': args_namespace.list,
                'access_levels': mappings.ACCESS_LEVELS_MAPPINGS['list'],
            },
            {
                'args': args_namespace.read,
                'access_levels': mappings.ACCESS_LEVELS_MAPPINGS['read'],
            },
            {
                'args': args_namespace.write,
                'access_levels': mappings.ACCESS_LEVELS_MAPPINGS['write'],
            },
            {
                'args': args_namespace.full_access,
                'access_levels': mappings.ACCESS_LEVELS_MAPPINGS['all'],
                'service_generator': generate_full_policy_for_service,
            },
        ]:

        if item['args']:
            for arg in item['args']:
                service_name = arg
                resource_type = '*'

                if ':' in arg:
                    service_name, resource_type = arg.split(':')

                if resource_type == '*':
                    if 'service_generator' in item:
                        policies.append(item['service_generator'](service_name))
                    else:
                        policies.append(generate_policy_for_service(service_name, item['access_levels']))
                else:
                    policies.append(generate_policy_for_service_arn_type(service_name, resource_type, item['access_levels']))

    if args_namespace.action:
        policies.append(create_policy(statement(actions=args_namespace.action, resource='*')))

    policy = simplify_policy(collapse_policy_statements(*policies))

    return policy



import argparse

parser = argparse.ArgumentParser(
    description="Generate IAM policies from the command line"
)
parser.add_argument(
    "-f",
    "--file",
    help="Specify a YAML file to process for policy generation",
)
parser.add_argument(
    "--max-length",
    type=int,
    default=6144,  # AWS max managed policy size
    help="Fail if policy length (in characters) exceeds this length.",
)
parser.add_argument(
    "--auto-shorten",
    action='store_true',
    default=False,
    help="Attempt to automatically shorten the policy if it's too long",
)
parser.add_argument(
    "-m",
    "--minimize",
    type=bool,
    default=False,
    help="Attempt to minimize policies",
)
parser.add_argument(
    "-c", "--compact",
    action="store_true",
    default=False,
    help="If specified, print compact JSON.",
)
parser.add_argument(
    "-A",
    "--action",
    action="append",
    help="Add a specific action to the policy",
)
parser.add_argument(
    "-a",
    "--full-access",
    action="append",
    help="Add full access for the specified services and resource type, which should be specified as service:type (e.g. ec2:instance) to the generated policy, can be repeated",
)
parser.add_argument(
    "-r",
    "--read",
    action="append",
    help="Add read access for the specified services and resource type, which should be specified as service:type (e.g. ec2:instance) to the generated policy, can be repeated",
)
parser.add_argument(
    "-w",
    "--write",
    action="append",
    help="Add write access for the specified services and resource type, which should be specified as service:type (e.g. ec2:instance) to the generated policy, can be repeated",
)
parser.add_argument(
    "-l",
    "--list",
    action="append",
    help="Add list access for the specified services and resource type, which should be specified as service:type (e.g. ec2:instance) to the generated policy, can be repeated",
)

